'use strict';

exports.__esModule = true;

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _reactDom = require('react-dom');

var _reactDom2 = _interopRequireDefault(_reactDom);

var _rcAlign = require('rc-align');

var _rcAlign2 = _interopRequireDefault(_rcAlign);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _CSSMotion = require('rc-animate/lib/CSSMotion');

var _CSSMotion2 = _interopRequireDefault(_CSSMotion);

var _motion = require('rc-animate/lib/util/motion');

var _reactLifecyclesCompat = require('react-lifecycles-compat');

var _PopupInner = require('./PopupInner');

var _PopupInner2 = _interopRequireDefault(_PopupInner);

var _LazyRenderBox = require('./LazyRenderBox');

var _LazyRenderBox2 = _interopRequireDefault(_LazyRenderBox);

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var Popup = function (_Component) {
  (0, _inherits3['default'])(Popup, _Component);

  function Popup(props) {
    (0, _classCallCheck3['default'])(this, Popup);

    var _this = (0, _possibleConstructorReturn3['default'])(this, _Component.call(this, props));

    _this.onAlign = function (popupDomNode, align) {
      var onAlign = _this.props.onAlign;
      // TODO: Double confirm this
      // const currentAlignClassName = props.getClassNameFromAlign(align);
      // FIX: https://github.com/react-component/trigger/issues/56
      // FIX: https://github.com/react-component/tooltip/issues/79
      // console.log('on align:', this.currentAlignClassName, currentAlignClassName);
      // if (this.currentAlignClassName !== currentAlignClassName) {
      //   this.currentAlignClassName = currentAlignClassName;
      //   popupDomNode.className = this.getClassName(currentAlignClassName);
      // }

      onAlign(popupDomNode, align);
    };

    _this.onMotionStart = function () {
      _this.setState({
        needAlign: false
      });
    };

    _this.getTargetElement = function () {
      return _this.props.getRootDomNode();
    };

    _this.getAlignTarget = function () {
      var point = _this.props.point;

      if (point) {
        return point;
      }
      return _this.getTargetElement;
    };

    _this.syncStretchSize = function () {
      var stretchUpdated = _this.state.stretchUpdated;
      var _this$props = _this.props,
          getRootDomNode = _this$props.getRootDomNode,
          stretch = _this$props.stretch;


      if (!stretch || stretchUpdated) return;

      var ele = getRootDomNode();

      var _ele$getBoundingClien = ele.getBoundingClientRect(),
          width = _ele$getBoundingClien.width,
          height = _ele$getBoundingClien.height;

      _this.setState({
        stretchUpdated: true,
        targetWidth: width,
        targetHeight: height
      });
    };

    _this.state = {
      // Used for stretch
      stretchUpdated: false,
      targetWidth: undefined,
      targetHeight: undefined,

      // Used for motion
      needAlign: true
    };

    _this.savePopupRef = _utils.saveRef.bind(_this, 'popupInstance');
    _this.saveAlignRef = _utils.saveRef.bind(_this, 'alignInstance');
    return _this;
  }

  Popup.prototype.componentDidMount = function componentDidMount() {
    this.rootNode = this.getPopupDomNode();
    this.syncStretchSize();
  };

  Popup.prototype.componentDidUpdate = function componentDidUpdate() {
    this.syncStretchSize();
  };

  /**
   * We manage the motion status by every props change:
   * 1. When visible: false -> true
   *    1. Reset needAlign: true
   *    2. Reset stretchUpdated if needed
   * 2. on componentDidUpdate
   *    1. if stretchUpdated is true, measure the size
   *      1. Set stretchUpdated to true
   * 3. Render popup but hidden it (visibility: false)
   *    1. Set needAlign: false
   *
   * Render visible:
   * 1. true until stretchUpdated & visible
   */
  Popup.getDerivedStateFromProps = function getDerivedStateFromProps(nextProps, _ref) {
    var _ref$prevProps = _ref.prevProps,
        prevProps = _ref$prevProps === undefined ? {} : _ref$prevProps;

    var newState = {
      prevProps: nextProps
    };

    if (nextProps.visible !== prevProps.visible && nextProps.visible) {
      newState.needAlign = true;
      newState.stretchUpdated = false;
    }

    return newState;
  };

  Popup.prototype.getPopupDomNode = function getPopupDomNode() {
    return _reactDom2['default'].findDOMNode(this.popupInstance);
  };

  // `target` on `rc-align` can accept as a function to get the bind element or a point.
  // ref: https://www.npmjs.com/package/rc-align


  Popup.prototype.getMaskTransitionName = function getMaskTransitionName() {
    var props = this.props;
    var transitionName = props.maskTransitionName;
    var animation = props.maskAnimation;
    if (!transitionName && animation) {
      transitionName = props.prefixCls + '-' + animation;
    }
    return transitionName;
  };

  Popup.prototype.getTransitionName = function getTransitionName() {
    if (!_motion.supportTransition) return null;

    var props = this.props;
    var transitionName = props.transitionName;
    if (!transitionName && props.animation) {
      transitionName = props.prefixCls + '-' + props.animation;
    }
    return transitionName;
  };

  Popup.prototype.getClassName = function getClassName(currentAlignClassName) {
    return this.props.prefixCls + ' ' + this.props.className + ' ' + currentAlignClassName;
  };

  Popup.prototype.getZIndexStyle = function getZIndexStyle() {
    var style = {};
    var props = this.props;
    if (props.zIndex !== undefined) {
      style.zIndex = props.zIndex;
    }
    return style;
  };

  // ============================== Dom ===============================


  // ============================= Render =============================
  Popup.prototype.renderPopupElement = function renderPopupElement() {
    var _this2 = this;

    var savePopupRef = this.savePopupRef;
    var _state = this.state,
        stretchUpdated = _state.stretchUpdated,
        targetHeight = _state.targetHeight,
        targetWidth = _state.targetWidth,
        needAlign = _state.needAlign;
    var _props = this.props,
        align = _props.align,
        visible = _props.visible,
        prefixCls = _props.prefixCls,
        style = _props.style,
        getClassNameFromAlign = _props.getClassNameFromAlign,
        destroyPopupOnHide = _props.destroyPopupOnHide,
        stretch = _props.stretch,
        children = _props.children,
        onMouseEnter = _props.onMouseEnter,
        onMouseLeave = _props.onMouseLeave,
        onMouseDown = _props.onMouseDown,
        onTouchStart = _props.onTouchStart;

    var className = this.getClassName(this.currentAlignClassName || getClassNameFromAlign(align));
    var hiddenClassName = prefixCls + '-hidden';

    if (!visible) {
      this.currentAlignClassName = null;
    }

    var sizeStyle = {};
    if (stretch) {
      // Stretch with target
      if (stretch.indexOf('height') !== -1) {
        sizeStyle.height = targetHeight;
      } else if (stretch.indexOf('minHeight') !== -1) {
        sizeStyle.minHeight = targetHeight;
      }
      if (stretch.indexOf('width') !== -1) {
        sizeStyle.width = targetWidth;
      } else if (stretch.indexOf('minWidth') !== -1) {
        sizeStyle.minWidth = targetWidth;
      }
    }

    var newStyle = (0, _extends3['default'])({}, sizeStyle, style, this.getZIndexStyle());

    var popupInnerProps = {
      prefixCls: prefixCls,
      ref: savePopupRef,
      onMouseEnter: onMouseEnter,
      onMouseLeave: onMouseLeave,
      onMouseDown: onMouseDown,
      onTouchStart: onTouchStart,
      style: newStyle,
      visible: visible
    };

    var mergedVisible = visible;
    if (stretch && !stretchUpdated) {
      mergedVisible = false;
    }

    var transitionName = this.getTransitionName();

    var mergedNeedAlign = visible && !transitionName;
    if (transitionName) {
      mergedNeedAlign = visible && needAlign;
    }

    return _react2['default'].createElement(
      _CSSMotion2['default'],
      {
        visible: mergedVisible,
        motionName: transitionName,
        removeOnLeave: destroyPopupOnHide,
        leavedClassName: hiddenClassName,
        onEnterStart: this.onMotionStart,
        onAppearStart: this.onMotionStart
      },
      function (_ref2) {
        var motionClassName = _ref2.className;

        return _react2['default'].createElement(
          _rcAlign2['default'],
          {
            target: _this2.getAlignTarget(),
            key: 'popup',
            ref: _this2.saveAlignRef,
            monitorWindowResize: true,
            disabled: !mergedNeedAlign,
            align: align,
            onAlign: _this2.onAlign
          },
          _react2['default'].createElement(
            _PopupInner2['default'],
            (0, _extends3['default'])({}, popupInnerProps, { className: (0, _classnames2['default'])(className, motionClassName) }),
            children
          )
        );
      }
    );
  };

  Popup.prototype.renderMaskElement = function renderMaskElement() {
    var _props2 = this.props,
        prefixCls = _props2.prefixCls,
        visible = _props2.visible,
        mask = _props2.mask;

    var maskElement = void 0;
    if (mask) {
      var maskTransition = this.getMaskTransitionName();

      var lazyRenderProps = {
        style: this.getZIndexStyle(),
        key: 'mask',
        className: prefixCls + '-mask',
        visible: visible
      };

      maskElement = _react2['default'].createElement(
        _CSSMotion2['default'],
        {
          motionName: maskTransition,
          leavedClassName: prefixCls + '-mask-hidden',
          visible: visible
        },
        function (_ref3) {
          var motionClassName = _ref3.className;

          return _react2['default'].createElement(_LazyRenderBox2['default'], (0, _extends3['default'])({}, lazyRenderProps, {
            className: (0, _classnames2['default'])(lazyRenderProps.className, motionClassName)
          }));
        }
      );
    }
    return maskElement;
  };

  Popup.prototype.render = function render() {
    return _react2['default'].createElement(
      'div',
      null,
      this.renderMaskElement(),
      this.renderPopupElement()
    );
  };

  return Popup;
}(_react.Component);

Popup.propTypes = {
  visible: _propTypes2['default'].bool,
  mask: _propTypes2['default'].bool,
  style: _propTypes2['default'].object,
  getClassNameFromAlign: _propTypes2['default'].func,
  onAlign: _propTypes2['default'].func,
  getRootDomNode: _propTypes2['default'].func,
  align: _propTypes2['default'].any,
  destroyPopupOnHide: _propTypes2['default'].bool,
  className: _propTypes2['default'].string,
  prefixCls: _propTypes2['default'].string,
  onMouseEnter: _propTypes2['default'].func,
  onMouseLeave: _propTypes2['default'].func,
  onMouseDown: _propTypes2['default'].func,
  onTouchStart: _propTypes2['default'].func,
  stretch: _propTypes2['default'].string,
  children: _propTypes2['default'].node,
  point: _propTypes2['default'].shape({
    pageX: _propTypes2['default'].number,
    pageY: _propTypes2['default'].number
  })
};


(0, _reactLifecyclesCompat.polyfill)(Popup);

exports['default'] = Popup;
module.exports = exports['default'];